var fs       = require('fs');
var path     = require('path');
var md       = require('markdown-it')();
var log      = require('npmlog');
var parser   = require('./parser');
var template = require('./template');
var read     = fs.readFileSync;
var exists   = fs.existsSync;

module.exports = tomdox;

// mixin parser API
Object.assign(tomdox, parser);

// mixin template generate method
Object.assign(tomdox, template);

// Public: Module entry point
//
// This method will parse the list of files, figure out each TomDoc section,
// build up a data Array for each file (which `--json` can output), compile
// Handlebars template and render the final result by writing each file to the
// destination directory.
//
// files      - The Array list of files to generate documentation from
// options    - Options hash with the following attributes
//    :output   - The String path to destination directory (default: docs)
//    :template - The String path to template directory with
//                index.html/file.html Handlebars template
//    :primary  - A String primary color to use with Material Design Lite (default: blue_grey)
//    :accent   - A String accent color to use with Material Design Lite (default: orange)
//    :json     - A Boolean to force raw output instead of writing files to disk
//    :debug    - A Boolean to prevent any operation on disk
//    :prefix   - A String to use when generating URLs (default: /)
// done       - An optional callback
//
// Returns a data object representing the parser result.
//
// Examples
//
//   var tomdox = require('tomdox');
//   var data = tomdox(['index.js', 'lib/foo.js', 'lib/bar.js'], {
//     debug: true
//   });
//   console.log(JSON.stringify(data, null, 2));
function tomdox(files, opts, done) {
  done = done || function() {};
  log.info('parser  ', 'Parsing %s files', files.length);
  log.info('parser  ', 'Files:', files.join('\n       '));

  files = files.map(function(f) {
    var content = read(f, 'utf8');
    var filename = path.resolve(f).replace(path.resolve() + '/', '');
    if (filename[0] === '/') filename = path.basename(f).replace(path.extname(f), '');

    return {
      filename: filename,
      content: content,
      file: filename,
      href: (opts.prefix || '/') + template.filename(filename) + '/index.html'
    };
  });

  var data = [];

  // Homepage
  data.push({
    file: 'index.html',
    filename: '',
    tomdocs: [],
    content: readme(),
    prefix: opts.prefix || '/',
    files: files
  });

  // Files
  data = data.concat(files.map((file) => {
    log.info('parser  ', 'Parsing file', file.filename);
    var lines = file.content.split(/\r?\n/);
    var comments = parser.comments(lines);
    var tomdocs = parser.tomdocs(comments);
    log.silly('tomdocs', tomdocs);

    return {
      file: path.basename(file.filename).replace(path.extname(file.filename), ''),
      filename: file.filename,
      tomdocs: tomdocs,
      content: file.content,
      prefix: opts.prefix || '/',
      files: files
    };
  }));

  template.generate(data, opts);

  return data;
}

function readme() {
  if (exists('readme.md')) return md.render(read('readme.md', 'utf8'));
  if (exists('README.md')) return md.render(read('README.md', 'utf8'));

  if (exists('readme.mkd')) return mkd.render(read('README.mkd', 'utf8'));
  if (exists('README.mkd')) return mkd.render(read('README.mkd', 'utf8'));

  if (exists('readme.markdown')) return markdown.render(read('README.markdown', 'utf8'));
  if (exists('README.markdown')) return markdown.render(read('README.markdown', 'utf8'));

  return defaultIndex();
}

function defaultIndex() {
  return `
    <h2>{{ name }}</h2>
    <ul class="mdl-list">
      {{#each files}}
      <li class="mdl-list__item">
        <span class="mdl-list__item-primary-content">
          <a href="{{ this }}">{{ this }}</a>
        </span>
      </li>
      {{/each}}
    </ul>
  `;
}
